@@ -14,14 +14,19 @@ import argparse
 import collections
 import cProfile as profile
 import functools
+import gettext
 import hashlib
 import json
+import locale
 import logging
 import logging.config
 import multiprocessing
 import os
-from os.path import abspath, basename, dirname, exists, isfile, join, splitext
+import subprocess
+from os.path import abspath, basename, dirname, expanduser, exists, isfile, join, splitext
 # from pprint import pprint, pformat
+import platform
+import plistlib
 import pstats
 # import subprocess
 import sys
@@ -29,6 +34,7 @@ import tempfile
 import threading
 import traceback
 from types import SimpleNamespace
+import uuid
 
 
 #
@@ -176,7 +182,7 @@ def build_default_logger(logdir, name=None, cfgfile=None, verbose=False):
     return logging.getLogger(name or 'default')
 
 
-_logger = build_default_logger(logdir=join(_script_dir, os.pardir, 'temp'))
+_logger = build_default_logger(logdir=join(_script_dir, os.pardir, 'temp'), name=splitext(basename(__file__))[0])
 
 
 def catch_unknown_exception(exc_type, exc_value, exc_traceback):
@@ -633,12 +639,104 @@ def get_local_tmp_dir():
         return join(os.getenv('LOCALAPPDATA'), 'Temp')
     elif plat == 'Darwin':
         return join(expanduser('~'), 'Library', 'Caches')
-    elif plat == 'Linux'
+    elif plat == 'Linux':
         return '/tmp'
     else:
         raise NotImplementedError(f'unsupported platform: {plat}')
 
 
+def write_plist_fields(cfg_file, my_map):
+    plist = {}
+    with open(cfg_file, 'rb') as fp:
+        plist = plistlib.load(fp, fmt=plistlib.FMT_XML)
+    plist.update(my_map)
+    with open(cfg_file, 'wb') as fp:
+        plistlib.dump(plist, fp)
+
+
+def substitute_keywords_in_file(file, str_map, useliteral=False):
+    updated = ''
+    with open(file) as f:
+        original = f.read()
+        if not useliteral:
+            updated = original % str_map 
+        else:
+            updated = original
+            for src, dest in str_map.items():
+                tmp = updated
+                updated = updated.replace(src, dest)
+    with open(file, 'w') as f:
+        f.write(updated)
+
+
+def is_uuid(text, version=4):
+    try:
+        uuid_obj = uuid.UUID(text, version=4)
+        return True
+    except ValueError:
+        return False
+
+
+def get_clipboard_content():
+    import tkinter as tk
+    root = tk.Tk()
+    # keep the window from showing
+    root.withdraw()
+    content = root.clipboard_get()
+    root.quit()
+    return content
+
+
+def alert(title, content, action='Close'):
+    if platform.system() == 'Windows':
+        cmd = ['mshta', f'vbscript:Execute("msgbox ""{content}"", 0,""{title}"":{action}")']
+    elif platform.system() == 'Darwin':
+        cmd = ['osascript', '-e', f'display alert "{title}" message "{content}"']
+    else:
+        cmd = ['echo', f'{title}: {content}: {action}']
+    subprocess.run(cmd)
+
+
+def convert_to_wine_path(path, drive='Z:'):
+    path = abspath(path).replace('/', '\\')
+    return drive + path
+
+
+def kill_process_by_name(name):
+    if platform.system() == 'Windows':
+        cmd = ['taskkill', '/IM', name, '/F']
+    else:
+        cmd = ['pkill', name]
+    subprocess.run(cmd)
+
+
+def init_translator(localedir, domain='all', langs=None):
+    """
+    - select locale and set up translator based on system language
+    - the leading language in langs, if any, is selected to override current locale
+    """
+    cur_langs = None
+    if langs:
+        cur_langs = langs
+    else:
+        cur_locale, encoding = locale.getdefaultlocale()
+        cur_langs = [cur_locale] if cur_locale else ['en']
+    translator = gettext.translation(domain, localedir=localedir, languages=cur_langs)
+    translator.install()
+    return translator.gettext
+
+
+def match_files_except_lines(file1, file2, excluded=None):
+    with open(file1) as fp:
+        content1 = fp.readlines()
+    with open(file2) as fp:
+        content2 = fp.readlines()
+    if excluded:
+        content1 = [cont for c, cont in enumerated(content1) if c not in excluded]
+        content2 = [cont for c, cont in enumerated(content2) if c not in excluded]
+    return content1 == content2
+
+
 class RerunLock:
     """Lock process from reentering when seeing lock file on disk."""
     def __init__(self, name, folder=None, infohook=_logger.info, warnhook=_logger.warning, errorhook=_logger.error):
@@ -654,18 +752,17 @@ class RerunLock:
                 pass
             return True
         else:
-            self.warnHook('Rerun disabled using file: {}. Unlock it before locking again. Aborted. You can ignore this message if your program is already running as planned.'.format(self.lockFile))
+            self.warnHook('Will not run the script while it is active with pid: {}.'.format(os.getpid()))
             return False
 
     def unlock(self):
         try:
             os.remove(self.lockFile)
-            self.infoHook('Rerun enabled after removing lock file: {}. You can ignore this message if your program finishes with success.'.format(self.lockFile))
         except FileNotFoundError:
-            self.warnHook('Failed to find lock file: {}. Rerun is enabled. Ignored.'.format(self.lockFile))
+            self.warnHook('Script reentrance is already enabled.')
         except Exception:
             failure = traceback.format_exc()
-            self.errorHook('{}\nFailed to enable rerun by deleting reentrance lock file: {}. You must delete it by hand to rerun your program. Ignored.'.format(failure, self.lockFile))
+            self.errorHook('{}\nFailed to unlock the script. You must delete the lock by hand: {}.'.format(failure, self.lockFile))
 
     def is_locked(self):
         return exists(self.lockFile)
@@ -677,13 +774,17 @@ def rerun_lock(name, folder=None, infohook=_logger.info, warnhook=_logger.warnin
         @functools.wraps(f)
         def wrapper(*args, **kwargs):
             my_lock = None
+            ret = None
             try:
                 my_lock = RerunLock(name, folder, infohook, warnhook, errorhook)
                 if not my_lock.lock():
                     return 1
                 ret = f(*args, **kwargs)
-            finally:  # Leave exception to global handler.
                 my_lock.unlock()
+            except Exception as e:  
+                my_lock.unlock()
+                # leave exception to its upper handler or let the program crash
+                raise e
             return ret
         return wrapper
     return decorator
